#
# spec file for package clusterctl
#
# Copyright (c) 2025 SUSE LLC
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


%define version_git_major 1
%define version_git_minor 8

Name:           clusterctl
Version:        1.9.4
Release:        1.1
Summary:        CLI tool to handle the lifecycle of a Cluster API management cluster
License:        Apache-2.0
URL:            https://github.com/kubernetes-sigs/cluster-api
Source:         %{name}-%{version}.tar.gz
Source1:        vendor.tar.gz
BuildRequires:  bash-completion
BuildRequires:  fish
BuildRequires:  go >= 1.23
BuildRequires:  zsh

%description
The clusterctl CLI tool handles the lifecycle of a Cluster API management
cluster.

The clusterctl command line interface is specifically designed for providing a
simple “day 1 experience” and a quick start with Cluster API. It automates
fetching the YAML files defining provider components and installing them.

Additionally it encodes a set of best practices in managing providers, that
helps the user in avoiding mis-configurations or in managing day 2 operations
such as upgrades.

Below you can find a list of main clusterctl commands:

* clusterctl init: Initialize a management cluster.
* clusterctl upgrade plan: Provide a list of recommended target versions for
  upgrading Cluster API providers in a management cluster.
* clusterctl upgrade apply: Apply new versions of Cluster API core and providers
  in a management cluster.
* clusterctl delete: Delete one or more providers from the management cluster.
* clusterctl generate: cluster Generate templates for creating workload
  clusters.
* clusterctl generate yaml: Process yaml using clusterctl’s yaml processor.
* clusterctl get kubeconfig: Gets the kubeconfig file for accessing a workload
  cluster.
* clusterctl move: Move Cluster API objects and all their dependencies between
  management clusters.
* clusterctl alpha rollout: Manages the rollout of Cluster API resources. For
  example: MachineDeployments.

Avoiding GitHub rate limiting

While using providers hosted on GitHub, clusterctl is calling GitHub API which
are rate limited; for normal usage free tier is enough but when using
clusterctl extensively users might hit the rate limit.

To avoid rate limiting for the public repos set the GITHUB_TOKEN environment
variable. To generate a token follow this documentation. The token only needs
repo scope for clusterctl.

%package -n %{name}-bash-completion
Summary:        Bash Completion for %{name}
Group:          System/Shells
Requires:       %{name} = %{version}
Requires:       bash-completion
Supplements:    (%{name} and bash-completion)
BuildArch:      noarch

%description -n %{name}-bash-completion
Bash command line completion support for %{name}.

%package -n %{name}-fish-completion
Summary:        Fish Completion for %{name}
Group:          System/Shells
Requires:       %{name} = %{version}
Supplements:    (%{name} and fish)
BuildArch:      noarch

%description -n %{name}-fish-completion
Fish command line completion support for %{name}.

%package -n %{name}-zsh-completion
Summary:        Zsh Completion for %{name}
Group:          System/Shells
Requires:       %{name} = %{version}
Supplements:    (%{name} and zsh)
BuildArch:      noarch

%description -n %{name}-zsh-completion
zsh command line completion support for %{name}.

%prep
%autosetup -p 1 -a 1

%build
COMMIT_HASH="$(sed -n 's/commit: \(.*\)/\1/p' %_sourcedir/%{name}.obsinfo)"

DATE_FMT="+%%Y-%%m-%%dT%%H:%%M:%%SZ"
BUILD_DATE=$(date -u -d "@${SOURCE_DATE_EPOCH}" "${DATE_FMT}" 2>/dev/null || date -u -r "${SOURCE_DATE_EPOCH}" "${DATE_FMT}" 2>/dev/null || date -u "${DATE_FMT}")
go build \
   -mod=vendor \
   -buildmode=pie \
   -ldflags=" \
   -X sigs.k8s.io/cluster-api/version.gitVersion=v%{version} \
   -X sigs.k8s.io/cluster-api/version.gitCommit=${COMMIT_HASH} \
   -X sigs.k8s.io/cluster-api/version.gitTreeState=clean \
   -X sigs.k8s.io/cluster-api/version.gitMajor=%{version_git_major} \
   -X sigs.k8s.io/cluster-api/version.gitMinor=%{version_git_minor} \
   -X sigs.k8s.io/cluster-api/version.gitReleaseCommit=${COMMIT_HASH} \
   -X sigs.k8s.io/cluster-api/version.buildDate=$BUILD_DATE" \
   -o bin/%{name} ./cmd/%{name}

%install
# Install the binary.
install -D -m 0755 bin/%{name} %{buildroot}/%{_bindir}/%{name}

# create the bash completion file
mkdir -p %{buildroot}%{_datarootdir}/bash-completion/completions/
%{buildroot}/%{_bindir}/%{name} completion bash > %{buildroot}%{_datarootdir}/bash-completion/completions/%{name}

# create the fish completion file
mkdir -p %{buildroot}%{_datarootdir}/fish/vendor_completions.d/
%{buildroot}/%{_bindir}/%{name} completion fish > %{buildroot}%{_datarootdir}/fish/vendor_completions.d/%{name}.fish

# create the zsh completion file
mkdir -p %{buildroot}%{_datarootdir}/zsh/site-functions/
%{buildroot}/%{_bindir}/%{name} completion zsh > %{buildroot}%{_datarootdir}/zsh/site-functions/_%{name}

%files
%doc README.md
%license LICENSE
%{_bindir}/%{name}

%files -n %{name}-bash-completion
%{_datarootdir}/bash-completion/completions/%{name}

%files -n %{name}-fish-completion
%{_datarootdir}/fish/vendor_completions.d/%{name}.fish

%files -n %{name}-zsh-completion
%{_datarootdir}/zsh/site-functions/_%{name}

%changelog
* Wed Jan 22 2025 opensuse_buildservice@ojkastl.de
- Update to version 1.9.4:
  * Update CAPO quick-start docs
  * More comments
  * More feedback
  * Address comments
  * Fix unit test
  * Add k/v pairs describing the overall status of the control
    plane
  * Improve Machine create and delete logs
  * [release-1.9] 🌱 inmemory: fix watch to continue serving based
    on resourceVersion parameter (#11710)
  * clustercache: Prevent concurrent map read/write when creating a
    cache
  * Address comments
  * Improve Machine remediation logs
  * Sort providers A-Z
  * Adds RKE2 to list of providers
  * Also patch external refs if the UID differs
  * Bump coredns support in docs
  * Bump github.com/coredns/corefile-migration to v1.0.25
  * patch: Call patchHelper only if necessary when reconciling
    external refs
  * patchHelper: call toUnstructured only if necessary
  * KCP: stop recreating logger for etcd client
  * Fix resync in in-memory provider
  * Fix formatting of blocking lifecycle hook condition message
  * Reduce log level of noisy logs in ExtensionConfig and topology
    Cluster controller
  * [release-1.9] 🌱 Add Infrastructure provider Harvester (#11653)
  * Fix broken links in the release team handbooks
  * Bump to controller-runtime v0.19.4
  * Bump envtest to v1.32.0
  * Fix log messages in Cluster set status code + some minor unit
    test fixes
  * fix a link for Cluster API videos and tutorials
  * Machine: sort list of pre-drain hooks for stable condition
    messages
* Mon Dec 23 2024 opensuse_buildservice@ojkastl.de
- Update to version 1.9.3:
  * Bump to golang.org/x/net v0.23.0
  * Export runtime.Client interface and cache package
* Sat Dec 21 2024 opensuse_buildservice@ojkastl.de
- Update to version 1.9.2:
  * [release-1.9] 🌱 Reconcile topology only when necessary
    (#11606)
  * [release-1.9] 🌱 Improve KCP scale up when using failure
    domains (#11604)
  * [release-1.9] 🌱 add TypedAll, ResourceIsUnchanged and
    TypedResourceIsUnchanged predicates (#11603)
  * Cache DiscoveryVariables calls
* Wed Dec 18 2024 opensuse_buildservice@ojkastl.de
- Update to version 1.9.1:
  * Update version page
  * CONTRIBUTING.md fix dead links
  * Bump golang.org/x/net
  * Bump golang.org/x/crypto
  * book: fix versions.md for v1.32
  * Set v1.7 EOL date and update CoreDns version table
  * Allow scalability to have custom deletion timeout
  * Bump default kind image to v1.32.0
  * machine: remove solved todo comment
  * Bump Kubernetes in tests to v1.32.0 and claim support for v1.32
  * 📖 Update clusterctl version to v1.9.x in quick start
  * 📖 add release 1.8 book link
* Wed Dec 11 2024 opensuse_buildservice@ojkastl.de
- Update to version 1.9.0:
  Full changelog is available here:
  https://github.com/kubernetes-sigs/cluster-api/releases/tag/v1.9.0
  * New providers in clusterctl: Canonical (#11337), Nutanix
    (#11135), Vultr (#11091)
  * New features:
  - clusterctl: Add support for v1beta2 conditions to clusterctl
    describe (#11277)
  - clusterctl: Allow user to suppress API warnings (#11149)
  - clusterctl: Suppress "finalizer name" API warnings in "move"
    command (#11173)
  * Bugfixes
  - clusterctl: Ensure move uses mutated metadata when updating a
    target object (#10957)
  - clusterctl: Fix: considers objects in kube-system for
    cert-manager to avoid upgrading twice (#11351)
  - clusterctl: Handle a nil mutator by returning an error, not
    panicking (#10951)
* Wed Dec 11 2024 opensuse_buildservice@ojkastl.de
- Update to version 1.8.6:
  * seedling: Add retry to clusterctl `UpgradeWithBinary`
  * Bump go to v1.22.10
  * Bump github.com/coredns/corefile-migration to v1.0.24
  * 🌱Update controller-runtime to v0.18.6
  * seedling: Add dry-run CreateOrUpdate call in clusterctl upgrade
    e2e tests
  * fix: do not reassign slice as it is used in different methods
  * fix: removes use of slices.DeleteFunc because it zeros the
    elements and doesn't remove those from obj list
  * fix: considers objects in kube-system for cert-manager to avoid
    upgrading twice
  * 🌱 [release-1.8] clusterctl: add nutanix ipam & runtime
    extensions providers
  * Uplift Go version to v1.22.9
* Tue Nov 12 2024 opensuse_buildservice@ojkastl.de
- Update to version 1.8.5:
  * Bump Go to v1.22.8 version
  * KCP: Skip validation if CoreDNS migration library supports an
    upgrade if skip CoreDNS is set
  * bump cert-manager to 1.16.1
  * seedling: Remove json logging replacement in e2e test config
  * test/framework: allow to include arbitrary types when dumping
    resources
  * fix: incorrect name used in kustomize path
  * kcp: consider all machines for setting .status.version
  * 🌱 test: add PreWaitForControlplaneToBeUpgraded to
    ClusterUpgradeConformanceSpec (#11145)
  * Update mailing list
* Fri Oct 11 2024 opensuse_buildservice@ojkastl.de
- Update to version 1.8.4:
  * Bump cert manager to v1.16.0
  * switch to using ECR mirror for trivy DB repo
  * add cacheSyncPeriod option to ClusterCacheTracker
  * pr-verify: use env var for passing the PR title
  * Update RKE2 provider URL
  * :seedling: test/e2e: decrease concurrency
  * [release-1.8] :seedling: Add vultr infrastructure provider
    (#11217)
  * test/framework: GetCAPIResources should warn on rbac issues
  * 🌱 test/framework: allow users to modify cache.Options
  * [release-1.8] 🌱 machine: prevent error spamming for
    NodeOutdatedTaint if objects are not found (#11199)
  * Reduce verbosity of logs when calling Runtime Extensions
  * clusterclass: fix nil pointer for empty workers in webhook
  * 📖 Drop v1.5 column, set v1.6 to EOL in versions.md
  * hack: fix the shell used when running pr-verify
  * [release-1.8] 🌱 replace kubebuilder-release-tools with new
    workflow (#11167)
* Sat Sep 14 2024 opensuse_buildservice@ojkastl.de
- Update to version 1.8.3:
  * Make KCP pre-terminate hook more robust
  * Bump to Go 1.22.7
  * Add more documentation about the KCP pre-terminate hook
  * test/e2e: increase concurrency
* Sat Sep 14 2024 opensuse_buildservice@ojkastl.de
- Update to version 1.8.2:
  * [release-1.8] 🐛 KCP: remove etcd member in pre-terminate hook
    (#11136)
  * book: document how to use Lima on macOS for development
  * 📖 Move Control Plane contracts doc to right chapter
  * re-add /finalizers subresource RBAC
  * test: Bump autoscaler to v1.31.0
  * 📖 Fix CAPZ redirection links in quick-start page
  * sorted labels and annotations in alphabatical order
  * [release-1.8] 🌱 test: improve autoscale tests for to/from zero
    and running autoscaler in bootstrap cluster (#11093)
  * update cert-manager to 1.15.3
  * Always declare try-or-die-command function in
    kubeadm-bootstrap-script.sh
  * Bump envtest in Makefile to v1.31.0
  * Bump kind to v0.24.0
  * docs: fix broken link
  * Update clusterctl version to v1.8.x in quick start
  * Set 1.6 EOL date
* Wed Aug 14 2024 opensuse_buildservice@ojkastl.de
- Update to version 1.8.1:
  * Bump Kubernetes in tests to v1.31.0 and claim support for v1.31
  * Improve clusterctl client config logic & error message
  * Update controller-runtime dependency to v0.18.5
* Tue Aug 13 2024 opensuse_buildservice@ojkastl.de
- Update to version 1.8.0:
  Changelog too big to include in toto, please check it here
  https://github.com/kubernetes-sigs/cluster-api/releases/tag/v1.8.0
  * Breaking Changes
  - ClusterClass: Adjust CC & Cluster controller to block on
    variable conflicts, deprecate definitionFrom (#10841)
  * Highlights
  - New providers in clusterctl: IONOS Cloud (#10767), Linode
    (#10471), Tinkerbell (#10487)
  * Bug fixes
  - clusterctl: Ensure cert-manager objects get applied before
    other provider objects (#10469)
  - clusterctl: Ensure move uses mutated metadata when updating a
    target object (#10982)
  - clusterctl: Fix log in clusterctl move (#10540)
  - clusterctl: Handle a nil mutator by returning an error, not
    panicking (#10981)
  - clusterctl: Verify that there is a release for the tag
    (#10220)
  * Others
  - clusterctl: Add ionoscloud provider (#10767)
  - clusterctl: Add support for the linode-linode infrastructure
    provider to clusterctl (#10471)
  - clusterctl: Add Tinkerbell to the providers list (#10487)
  - clusterctl: Always run crd migration if possible to reduce
    conversion webhook usage (#10513)
  - clusterctl: Bump cert-manager to 1.14.5 (#10514)
  - clusterctl: Bump cert-manager to v1.15.0 (#10727)
  - clusterctl: Bump cert-manager to v1.15.1 (#10807)
  - clusterctl: Bump cert-manager to v1.15.2 (#10992)
  - clusterctl: Improve cert-manager shouldUpgrade (#10407)
  - clusterctl: Update Config generation to use inClusterConfig
    (#11006)
  - Testing: Allow control plane count to be configurable in
    clusterctl upgrade test. (#10584)
  - Testing: Support MachinePools without MachinePoolMachines in
    clusterctl upgrade test (#10498)
* Mon Aug 12 2024 opensuse_buildservice@ojkastl.de
- Update to version 1.7.5:
  * Machine Controller should try to retrieve node on delete - fix
    unit test
  * bug: Machine Controller should try to retrieve node on delete
  * MachineSet should allow scale down operations to proceed
    when templates don't exist
  * Bump docker/docker to v25.0.6
  * test/framework: scale up should use allocatable memory
  * Ensure DockerMachinePool providerIDList is deterministic
  * test: prevent using Consistently for rollout checks
  * test/framework: Allow ScaleUp Deployment image to be customized
  * bump cert-manager to 1.15.2
  * Drop caBundle from CRDs
  * Allow ClusterCacheTracker to set CacheByObject
  * bug: When infrastructureRef is nil, set
    InfrastructureReadyCondition to true
  * Partially revert changes for ":bug: Cluster should be
    provisoned when cpRef and endpoint is set"
  * :seedling: Make ClusterResourceSet controller more predictable
  * :sparkles: Add QPS & burst options & flags for
    ClusterCacheTracker
  * Address comments
  * Fix race conditions
  * Address comments
  * Ensure CRS controller always add ownerReference to resources
  * test: use pause instead of nginx
  * test: use pause image for autoscaler tests
  * bug: Cluster should be provisoned when cpRef and endpoint is
    set
  * capd: fix nil pointer in dockermachinepool controller
  * book: Add missing kubeconfig file to deploy
    cloud-controller-manager on workload cluster
  * book: update Azure quick start for CAPZ v1.16
* Wed Jul 10 2024 opensuse_buildservice@ojkastl.de
- Update to version 1.7.4:
  * Extend ControlPlane/Deployment/Pool builtin to include metadata
  * [release-1.7] ✨ Allow CAPBK to generate JoinConfiguration
    discovery kubeconfig (#10842)
  * bug: Patch helper should be able to patch non-spec objects
  * Extend cluster builtin to include UID
  * Bump to Go 1.22.5
  * [release-1.7] 🌱 Allow control plane provider to set endpoint
    (#10800)
  * bump cert-manager to 1.15.1
  * Bump to Go 1.22.4
  * docs: fix dead link in
    proposals/20210222-kubelet-authentication.md
  * 🐛 Correctly handle concurrent updates to
    ClusterResourceSetBinding
  * MD/MS topo reconciler: only add finalizer for owned MD/MS
  * Update quick-start.md
  * Update Proxmox quick-start docs
  * ✨ Add kubeadm v1beta4 types (#10709)
  * Deprioritize unknown NodeHealthy conditions for deletion
  * [release-1.7] :seedling: Add ionoscloud provider (#10769)
  * fix: delayed MHC replacement of unreachable nodes
  * fix(10661): volumes don't block deletion of unreachable nodes
  * Improve logs when machine is waiting for node
  * Ensure unhealthy machines get deletion priority
  * book: Fix comment on NodeStartupTimeout
  * :sparkles: Add remediation strategy support for MachineSet
  * seedling: MachineHealthCheck should take Machine's InfraReady
    condition for startup time
  * [release-1.7] 🌱 Improvements to ownerReferences and finalizers
    test (#10747)
* Sun Jun 16 2024 opensuse_buildservice@ojkastl.de
- Update to version 1.7.3:
  * Bump to Go 1.21.11
  * update cert-manager to v1.15.0
  * fix broken kubebuilder link in old docs
  * Support ControlPlane variable overrides
  * docs: fix dead links
* Sun Jun 16 2024 Johannes Kastl <opensuse_buildservice@ojkastl.de>
- new package clusterctl: tool that handles the lifecycle of a
  Cluster API management cluster
